"use strict";

var OO = require('../basics/oo');
var Component = require('./component');
var $$ = Component.$$;

function DropdownComponent() {
  Component.apply(this, arguments);
}

DropdownComponent.Prototype = function() {

  this.getInitialState = function() {
    return {
      open: false
    };
  };

  // Note: It's important that all children tools are rendered (even if not shown)
  // because only that way we can keep the disabled states accurate
  this.render = function() {
    var el = $$('div').addClass('dropdown');
    if (this.state.open) {
      el.addClass('open');
    }
    el.append(
      $$('button').key('toggle')
        .addClass('toggle')
        .attr('title', this.props.title)
        .append(this.props.label)
        .on('mousedown', this.handleDropdownToggle)
        .on('click', this.handleClick),
      $$('div').key('options')
        .addClass('options shadow border fill-white')
        .append(this.props.children)
    );
    return el;
  };

  // Prevent click behavior as we want to preserve the text selection in the doc
  this.handleClick = function(e) {
    e.preventDefault();
  };

  this.handleDropdownToggle = function(e) {
    e.preventDefault();
    var open = this.state.open;
    var self = this;
    if (open) return;
    this.setState({open: !this.state.open});
    setTimeout(function() {
      $(window).one('mousedown', function(e) {
        /*jshint unused: false */
        // e.preventDefault();
        // e.stopPropagation();
        self.close();
      });
    }, 0);
  };

  this.close = function() {
    this.setState({
      open: false
    });
  };
};

OO.inherit(DropdownComponent, Component);

module.exports = DropdownComponent;
